function makeModel(CP::ControlParameters; rstar = 0.0639381878390887)
  dm, alpha0, alpha1, s, cbar, sigmal, hbar, mu_male, mu_female = CP.dm, CP.alpha0, CP.alpha1, CP.s, CP.cbar, CP.sigmal, CP.hbar, CP.mu_male, CP.mu_female
  #grid_n = 250
  #grid_n = 700
  grid_n = 1700
  bign = 40*grid_n
  lb = 0.
  ub = 8000.
  #ub = 10.
  grid = loggrid(lb, ub, grid_n)
  biga_grid = loggrid(lb, ub, bign)
  #grid = [collect(linspace(lb, 10., grid_n1)) ; collect(linspace(10.1, ub, grid_n2)) ]

  alpha = 0.376
  dep = 0.059
  PT = ProductionTechnology(alpha, dep)
  beta = 0.924
  #beta = 0.96
  omega = 0.5 # Transfers to retired
  HB = HouseholdBehavior(beta, omega)
  sigmac = 1.5
  #sigmac = 1.458

  # Average female labor is 1.2
  # Target female elasticity is 0.85
  # if E = (3.2 - l)/l * (1/theta), then
  sigmal = sigmal
  #sigmal = 1.016
  #phi = 1.138
  hoursmax = 3.2
  df = alpha0 * dm.^alpha1
  hbar = hbar
  UF = CRRA_Family(sigmac, dm, df, sigmal, hbar, hoursmax)
  #UF = CRRA(sigmac, df, sigmal, hbar, hoursmax)

  Gee=1/100* (
  [96.24 1.14 0.39 0.006 ;
  3.07 94.33 0.37 0;
  1.5 0.43 95.82 0.02;
  10.66 0.49 6.11 80.51]; )
  Gee=Gee-diagm(sum(Gee, 2)[:] - 0.9778) #Fix rounding errors from the paper
  Pi_sp_EE = Gee ./ sum(Gee, 2)
  Pi_sp_ER = eye(4)
  Pi_sp_RR = eye(4)
  Gre = zeros(4,4)

  statE=[61.11;22.35;16.5;0.0389]/100;
  statE=statE/sum(statE);
  phi1=0.969;
  phi2=0.525;
  Gre[1:1,1]=[1 phi1 phi1^2 phi1^3]*statE;
  Gre[1:1, 2]=[0 1-phi1 (1-phi1)*phi1 (1-phi1)*phi1^2]*statE;
  Gre[1:1, 3]=(1-phi1)*[0 0 1 phi1]*statE;
  Gre[1:1, 4]=(1-phi1)*[0 0 0 1]*statE;
  Gre[2:2, 1]=(1-phi1)*[1 0 0 0]*statE;
  Gre[2:2, 2]=[phi1 1 phi1 phi1^2]*statE;
  Gre[2:2, 3]=(1-phi1)*[0 0 1 phi1]*statE;
  Gre[2:2, 4]=(1-phi1)*[0 0 0 1]*statE;
  Gre[3:3, 1]=(1-phi1)*[1 0 0 0]*statE;
  Gre[3:3, 2]=(1-phi1)*[phi1 1 0 0]*statE;
  Gre[3:3, 3]=[phi1^2 phi1 1 phi1]*statE;
  Gre[3:3, 4]=(1-phi1)*[0 0 0 1]*statE;
  Gre[4:4, 1]=(1-phi1)*[1 0 0 0]*statE;
  Gre[4:4, 2]=(1-phi1)*[phi1 1 0 0]*statE;
  Gre[4:4, 3]=(1-phi1)*[phi1^2 phi1 1 0]*statE;
  Gre[4:4, 4]=[phi1^3 phi1^2 phi1 1]*statE;

  tempGre=zeros(4, 4);
  for i=1:4
    tempGre[i,1]=Gre[i,1]+phi2*Gre[i,2]+phi2^2*Gre[i,3]+phi2^3*Gre[i,4];
    tempGre[i,2]=(1-phi2)*(Gre[i,2]+phi2*Gre[i,3]+phi2^2*Gre[i,4]);
    tempGre[i,3]=(1-phi2)*(Gre[i,3]+phi2*Gre[i,4]);
    tempGre[i,4]=(1-phi2)*Gre[i,4];
  end
  Pi_sp_RE = tempGre

  Pi_s = zeros(4,4, 2, 2, length(dm))
  Pi_s[:, :, 1, 1, 1] = Pi_sp_EE
  Pi_s[:, :, 1, 2, 1] = Pi_sp_ER
  Pi_s[:, :, 2, 1, 1] = Pi_sp_RE
  Pi_s[:, :, 2, 2, 1] = Pi_sp_RR
  for i = 1:length(dm)
    Pi_s[:, :, :, :, i] = Pi_s[:, :, :, :, 1]
  end


  rho_re=0.0222;
  #rho_re = 0.
  rho_rr=1-0.066;
  #rho_rr = 1.
  Pi_r = [1 - rho_re rho_re ;
  1 - rho_rr rho_rr]


  lambda_e_m = 0.98
  lambda_n_m = 0.2824
  lambda_e_f = 0.98
  lambda_n_f = 0.2824
  Pi_emp_m = zeros(2, 2)
  Pi_emp_m[:, 1] = [lambda_e_m ; 1 - lambda_e_m]
  Pi_emp_m[:, 2] = [1 - lambda_n_m; lambda_n_m]
  Pi_emp_f = zeros(2, 2)
  Pi_emp_f[:, 1] = [lambda_e_f ; 1 - lambda_e_f]
  Pi_emp_f[:, 2] = [1 - lambda_n_f; lambda_n_f]


  s_vals1 = [1; 3.15; 9.78; 1061]
  s_vals = zeros(length(s_vals1), length(dm))
  for i in 1:length(dm)
    s_vals[:, i] = s[i]*s_vals1
  end
  clamp!(s_vals, 0., 1061)
  #mu_female = 0.
  # Max average female hours is about 1.7. Call that full time male hours
  # 1.7/3.2 is 0.53
  mu_male = mu_male # CDR target working 30% of the 3.2 total disposible time
  mu_female = mu_female # Females earn 77%
  IP = IncomeProcess(Pi_s, Pi_r, Pi_emp_m, Pi_emp_f, s_vals, mu_male, mu_female, cbar)


  a0=.258;
  a1=.768;
  a2=.491;
  a3=.144;
  zlower=14.101;
  tauE=.16;

  estatetaxes = EstateTaxes(zlower, tauE)
  incometaxes = IncomeTaxes(a0, a1, a2, a3)

  #as = AggState(0.06, 1.2481)
  as = AggState(rstar, 1.2321462252537372)
  budget_hours_minconsump_guess_NW = ones(grid_n, size(s_vals, 1), 2, 2, 2, length(dm)) * hbar
  VFsize = size(budget_hours_minconsump_guess_NW)
  budget_hours_minconsump_guess_WW = ones(VFsize) * hbar
  budget_hours_guess_NW = ones(VFsize)
  budget_hours_guess_WW = ones(VFsize)
  estate_send_nx = ones(Int, grid_n)
  bigestate_send_nx = ones(Int, bign)
  brentguess = zeros(VFsize..., 4) # 4 is the number of work status pairs
  IS_array = Array(IdioState, VFsize)
  dumITP = interpolate((grid,), grid, Gridded(Linear()))
  etc = Dict([(:T, 72)])
  storage = Storage(budget_hours_minconsump_guess_NW, budget_hours_minconsump_guess_WW,
  budget_hours_guess_NW, budget_hours_guess_WW,
  brentguess, estate_send_nx, bigestate_send_nx, VFsize, IS_array, dumITP, etc)
  model = Model(UF, IP, HB, PT, incometaxes, estatetaxes, as, grid, biga_grid, storage, CP)
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = VFsize
  for s_nx in 1:n_s, ret_nx in 1:n_ret, emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d
    for a_nx in 1:n_a
      model.storage.IS_array[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx] = buildState(a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx, model)
    end
  end

  estate_send_nx = locateslow2(Float64[a - estatetax(a, model) for a in grid], grid)
  model.storage.estate_send_nx[:] = estate_send_nx[:]
  bigestate_send_nx = locateslow2(Float64[a - estatetax(a, model) for a in biga_grid], biga_grid)
  model.storage.bigestate_send_nx[:] = bigestate_send_nx[:]
  itp = buildincometax2(model)
  model.storage.itp = itp
  return model
end
