

function bellman_obj(ap::Number, state::State, EVspline, model::Model,
  policy::PolicyFunctions, OO::WorkStatus)
  -( solve_intra(state, ap, model, policy, OO) + model.HB.beta * EVspline(ap))
end
Base.call{T<:Number}(f::pars_bellman_obj, ap::T) = bellman_obj(ap, f.state, f.EVspline, f.model, f.policy, f.OO)::T



function solve_intra(state::State, ap::Number, model::Model, policy::PolicyFunctions,
  OO::WorkStatus; update_policy::Bool = false)
  mhours, fhours = getHours(state, ap, model, policy, OO)
  c = budget_c_given_h(mhours, fhours, state, ap, model) # Basic is by default TRUE
  y = aftertax_income_fun(mhours, fhours, state, ap, model)
  if update_policy
    nx = state.IS.nx
    policy.g_c[nx] = c
    policy.gh_m[nx] = mhours
    policy.gh_f[nx] = fhours
    policy.g_income[nx] = y
    return zero(ap)
  end
  return U(c, mhours, fhours, state, model)
end

# This is a bit of a hack to make male working impossible when there is no family (UF = CRRA)
# When UF is CRRA and OO indicates the male will work, generate -Inf utility
function solve_intra{A <: AnAggState}(state::State, ap::Number, model::Model{CRRA, A}, policy::PolicyFunctions,
  OO::Union{WN, WW}; update_policy::Bool = false)
  return -Inf
end

function solve_intra_nosolve(state::State, ap::Number, model::Model, policy::PolicyFunctions)
  nx = state.IS.nx
  mhours, fhours = policy.gh_m[nx], policy.gh_f[nx]
  c = policy.g_c[nx]
  return U(c, mhours, fhours, state, model)
end

function getHours(state::State, ap::Number, model::Model, policy::PolicyFunctions,
  OO::NN)
  return (0., 0.)
end

function getHours(state::State, ap::Number, model::Model, policy::PolicyFunctions,
  OO::WN)
  hoursmax = model.UF.hoursmax
  return (hoursmax, 0.)
end

mhours_fun(::Union{NW, NN}, model::Model) = 0.
mhours_fun(::Union{WN, WW}, model::Model) = model.UF.hoursmax
function getHours(state::State, ap::Number, model::Model, policy::PolicyFunctions,
  OO::Union{NW, WW})
  hbar = model.UF.hbar
  hoursmax = model.UF.hoursmax
  mhours = mhours_fun(OO, model)
  IS = state.IS
  nx = IS.nx
  f = pars_hoursFOC(mhours, state, ap, model, OO)
  if f(hbar) < 0. # Min marginal utility of leisure exceeds marginal utility of consumption
    fhours = 0. # Corner
  elseif  budget_c_given_h(mhours, hoursmax, state::State, # Use basic = false since this is for the FOC
    ap::Number, model::Model, false) <= 0. # Illegal assets, requires negative consumption
    # Isn't this redundant? I check the max aprime for positive consumption before coming here
    # No, when a = 0 and females are unproductive, this can happenn
    fhours = hoursmax # So, this generates -Inf utility
  else
    f(3.0) > 0 && return (mhours, hoursmax) # This choice requires extreme labor supply and would never be chosen, so abort it
    # The nonlinear solver is unstable at these values
    minH = budget_hours_minconsump_fun(mhours, state, ap, model)
    if mhours == 0.
    #  fhoursguess = model.storage.budget_hours_guess_NW[nx]
      fhoursguess = 0.8 * minH + 0.2 * hoursmax
      fhours = myNewton2(f, fhoursguess, minH, hoursmax)
    #  model.storage.budget_hours_guess_NW[nx] = fhours
    else
    #  fhoursguess = model.storage.budget_hours_guess_WW[nx]
      fhoursguess = 0.8 * minH + 0.2 * hoursmax
      fhours = myNewton2(f, fhoursguess, minH, hoursmax)
    # model.storage.budget_hours_guess_WW[nx] = fhours
    end
  end
  return (mhours, fhours)
end

function testgetHours(state::State, ap::Number, model::Model, policy::PolicyFunctions,
  OO::Union{NW, WW})
  function UoH(fhours)
    c = budget_c_given_h(mhours::Number, fhours::Number,
    state::State, ap::Number, model::Model, false)
    -U(c, mhours, fhours, state, model)
  end
  hbar = model.UF.hbar
  hoursmax = model.UF.hoursmax
  mhours = mhours_fun(OO, model)
  f = pars_hoursFOC(mhours, state, ap, model, OO)
  if f(hbar) < 0. # Min marginal utility of leisure exceeds marginal utility of consumption
    fhours = 0. # Corner
  else
    fhours = brent(UoH, hbar, hoursmax)[1]
  end
  return (mhours, fhours)
end


function hoursFOC(mhours::Number, fhours::Number, state::State, ap::Number, model::Model,
  OO::Union{NW, WW})
  mhours = mhours_fun(OO, model)
  # DcDh automatically ignores the basic income for the FOC
  c, w = DcDh(mhours, fhours, state, ap, model) # Autodiff returns both, more efficient
  w * uc(c, model) - ul(fhours, state, model)
end
# Not sure why this works but it fixed some type instability
Base.call{T<:Number}(f::pars_hoursFOC, hours::T) = hoursFOC(
f.mhours, hours, f.state, f.ap, f.model, f.OO)::T

function U{A <: AnAggState}(c::Number, mhours::Number, fhours::Number,
  state::State, model::Model{CRRA_Family, A})
  UF = model.UF
  hoursmax = UF.hoursmax

  (c > 0. && fhours < hoursmax) || return -Inf

  d_m = state.IS.d_m
  d_f = state.IS.d_f
  sigmac = UF.sigmac
  psi_f = UF.psi_f
  (c^(1 - sigmac)/(1-sigmac) + d_m*(hoursmax - mhours)
  + d_f *( (hoursmax - fhours)^(1 - psi_f) - 1) /(1 - psi_f)  )
end

function uc{A <: AnAggState}(c::Number, model::Model{CRRA_Family, A})
  c < zero(c) && return Inf*one(c)
  return c^(-model.UF.sigmac)
end
function ul{A <: AnAggState}(fhours::Number, state::State, model::Model{CRRA_Family, A})
  UF = model.UF
  hoursmax = UF.hoursmax
  psi_f = UF.psi_f
  fhours > hoursmax && return Inf*one(fhours)
  d_f = state.IS.d_f
  return d_f * (hoursmax - fhours)^-psi_f
end

function U{A <: AnAggState}(c::Number, mhours::Number, fhours::Number,
  state::State, model::Model{CRRA, A})
  UF = model.UF
  hoursmax = UF.hoursmax

  (c > 0. && fhours < hoursmax) || return -Inf

  d_f = state.IS.d_f
  sigmac = UF.sigmac
  psi_f = UF.psi_f

  c^(1 - sigmac)/(1-sigmac) + d_f *(hoursmax - fhours)^(1 - psi_f)/(1 - psi_f)
end

function uc{A <: AnAggState}(c::Number, model::Model{CRRA, A})
  c < zero(c) && return Inf*one(c)
  return c^(-model.UF.sigmac)
end
function ul{A <: AnAggState}(fhours::Number, state::State, model::Model{CRRA, A})
  UF = model.UF
  hoursmax = UF.hoursmax
  psi_f = UF.psi_f
  fhours > hoursmax && return Inf*one(fhours)
  d_f = state.IS.d_f
  return d_f * (hoursmax - fhours)^-psi_f
end
