using Debug
using Dierckx
using ForwardDiff
using Interpolations
using Roots
using Optim
using JLD
#include("cubic_splines.jl")
# indexed by (assets_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx)



type Aggregates
  A::Float64
  Lm::Float64
  Lf::Float64
  C::Float64
  L::Float64
  Hm::Float64
  Hf::Float64
  H::Float64
  Y::Float64
  Y_prod::Float64
  G::Float64
end

type ControlParameters
  dm::Vector{Float64}
  pd1::Float64
  pd3::Float64
  ps1d1::Float64
  ps2d2::Float64
  ps3d3::Float64
  alpha0::Float64
  alpha1::Float64
  s::Vector{Float64}
  cbar::Float64
  sigmal::Float64
  hbar::Float64
  mu_male::Float64
  mu_female::Float64
end

type IncomeProcess
  Pi_s::Array{Float64, 5} # Index s, s', ret, ret', d. P(s' | s, ret, ret', d)
  Pi_ret::Array{Float64, 2} # Index ret, ret'
  Pi_emp_m::Array{Float64, 2} # Index emp', labor
  Pi_emp_f::Array{Float64, 2} # Index emp', labor
  s_vals::Array{Float64, 2}
  mu_male::Float64
  mu_female::Float64
  cbar::Float64
end

type HouseholdBehavior
  beta::Float64
  omega::Float64
end
type ProductionTechnology
  alpha::Float64
  delta::Float64
end

type IncomeTaxes
  a0::Float64
  a1::Float64
  a2::Float64
  a3::Float64
end


type EstateTaxes
  zlower::Float64
  tauE::Float64
end

immutable PolicyFunctions{T1 <: AbstractArray{Float64}, T2 <: AbstractArray{Int}}
  g::T2
  gval::T1
  g_c::T1
  gh_m::T1
  gh_f::T1
  g_worktype_nx::T2
  g_income::T1
end


abstract UtilityFunction

type CRRA_Family <: UtilityFunction
  sigmac::Float64
  d_m::Vector{Float64}
  d_f::Vector{Float64}
  psi_f::Float64
  hbar::Float64
  hoursmax::Float64
end

type CRRA <: UtilityFunction
  sigmac::Float64
  d_f::Vector{Float64}
  psi_f::Float64
  hbar::Float64
  hoursmax::Float64
end

type FamilyIndivisibleLabor <: UtilityFunction
  # is log consumption
  d_m::Vector{Float64}
  d_f::Vector{Float64}
  hoursmax::Float64
  hbar::Float64
end

abstract AnAggState

immutable AggState <: AnAggState
  r::Float64
  w::Float64
end

immutable AggStateSticky <: AnAggState
  r::Float64
  w::Float64
  profits::Float64
end

type StickParams
  theta::Float64
  rho::Float64
end
# indexed by (assets_nx, s_nx, ret_nx, emp_nx, d_nx)
immutable IdioState
  a::Float64
  s::Float64
  d_m::Float64
  d_f::Float64
  ret::Bool # TRUE if retired
  unemp_m::Bool # TRUE if unemployed
  unemp_f::Bool
  a_nx::Int
  s_nx::Int
  d_nx::Int
  nx::Int # The sub2ind version
end

immutable State{T1 <: IdioState, T2<:AnAggState}
  IS::T1
  AS::T2
end

type Storage
  budget_hours_minconsump_guess_NW::Array{Float64, 6}
  budget_hours_minconsump_guess_WW::Array{Float64, 6}
  budget_hours_guess_NW::Array{Float64, 6}
  budget_hours_guess_WW::Array{Float64, 6}
  brentguess::Array{Float64, 7}
  estate_send_nx::Vector{Int} # Index of closest gridpoint after estate tax
  bigestate_send_nx::Vector{Int} # Index of closest gridpoint after estate tax
  VFsize::Tuple{Int64,Int64,Int64,Int64,Int64, Int64}
  IS_array::Array{IdioState, 6}
  itp::Interpolations.GriddedInterpolation{Float64,1,Float64,Interpolations.Gridded{Interpolations.Linear},Tuple{Array{Float64,1}},0}
  etc::Dict{Symbol, Float64}
end

type Model{TU<:UtilityFunction, TA <: AnAggState}
  UF::TU
  IP::IncomeProcess
  HB::HouseholdBehavior
  PT::ProductionTechnology
  IT::IncomeTaxes
  ET::EstateTaxes
  AST::TA
  a_grid::Vector{Float64}
  biga_grid::Vector{Float64}
  storage::Storage
  CP::ControlParameters
end

type SteadyState
  VF::Array{Float64, 6}
  policy::PolicyFunctions
  bigpolicy::PolicyFunctions
  dist::Array{Float64, 6}
  aggs::Aggregates
  r::Float64
end

abstract WorkStatus
immutable NN <: WorkStatus end
immutable NW <: WorkStatus end
immutable WN <: WorkStatus end
immutable WW <: WorkStatus end

immutable pars_budget_c_given_h{StateT <: State, T <: Number, ModelT <: Model}
  state::StateT
  ap::T
  model::ModelT
  mhours::Float64
  basic::Bool
end

immutable pars_bellman_obj{StateT <: State, ModelT <: Model, PolicyT <: PolicyFunctions,
                           T <: Any, WorkStatusT <: WorkStatus}
  state::StateT
  EVspline::T # Is nowork spline
  model::ModelT
  policy::PolicyT
  OO::WorkStatusT
end

immutable pars_hoursFOC{StateT <: State, T <: Number, ModelT <: Model, WorkStatusT <: WorkStatus}
  mhours::T
  state::StateT
  ap::T
  model::ModelT
  OO::WorkStatusT
end
