# Generic bellman operator for a given beta, value function, probability distribution,
# and current-period utility grid



# Compute EV(a' | state). Produce vector over a'

function compute_expectation!(s_nx, ret_nx, d_nx, work_m::Bool, work_f::Bool, VF::AbstractArray,
  model::Model, EV::AbstractVector, emp_m_nx, emp_f_nx)
  const ENDO_UNEMP = true
  IP = model.IP
  # Really this should use interpolation, but this is faster
  estate_send_nx::Vector{Int} = model.storage.estate_send_nx
  work_m_nx = work_m ? 1 : 2
  work_f_nx = work_f ? 1 : 2
  Pi_s::Array{Float64, 5} = IP.Pi_s # Index s, s', ret, ret', d. P(s' | s, ret, ret', d)
  Pi_ret::Array{Float64, 2} = IP.Pi_ret # Index ret, ret'
  Pi_emp_m::Array{Float64, 2} = IP.Pi_emp_m # Index emp', labor
  Pi_emp_f::Array{Float64, 2} = IP.Pi_emp_f
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = model.storage.VFsize
  @inbounds for ap_nx = 1:n_a
    ev = 0.
    @inbounds for sp_nx = 1:n_s, emp_mp_nx=1:n_emp_m, emp_fp_nx = 1:n_emp_f, retp_nx = 1:n_ret
      prob_sp = Pi_s[s_nx, sp_nx, ret_nx, retp_nx, d_nx]
      prob_rp = Pi_ret[ret_nx, retp_nx]
      if ENDO_UNEMP
        prob_emp_mp = Pi_emp_m[emp_mp_nx, work_m_nx]
        prob_emp_fp = Pi_emp_f[emp_fp_nx, work_f_nx]
      else
        prob_emp_mp = Pi_emp_m[emp_mp_nx, emp_m_nx]
        prob_emp_fp = Pi_emp_f[emp_fp_nx, emp_f_nx]
      end
      if prob_sp == 0 || prob_rp == 0 || prob_emp_mp == 0 || prob_emp_fp ==0
        continue
      end
      if retp_nx == 1 && ret_nx == 2 # If being reborn
        val = VF[estate_send_nx[ap_nx], sp_nx, retp_nx, 1, 1, d_nx] # Apply estate tax and force employment
      else
        val = VF[ap_nx, sp_nx, retp_nx, emp_mp_nx, emp_fp_nx, d_nx]
      end
      prob = prob_sp * prob_rp * prob_emp_mp * prob_emp_fp
      ev += prob * val
    end
    EV[ap_nx] = ev
  end
end



function buildState{A <: AnAggState}(a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx, model::Model{CRRA_Family, A})
  a = model.a_grid[a_nx]
  s = model.IP.s_vals[s_nx, d_nx]
  UF = model.UF
  d_m = UF.d_m[d_nx]
  d_f = UF.d_f[d_nx]
  ret = ret_nx==2
  unemp_m = emp_m_nx==2
  unemp_f = emp_f_nx==2
  nx = sub2ind(model.storage.VFsize, a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx)
  state = IdioState(a, s, d_m, d_f, ret, unemp_m, unemp_f, a_nx, s_nx, d_nx, nx)
end

function buildState{A <: AnAggState}(a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx, model::Model{CRRA, A})
  a = model.a_grid[a_nx]
  s = model.IP.s_vals[s_nx, d_nx]
  UF = model.UF
  d_m = Inf
  d_f = UF.d_f[d_nx]
  ret = ret_nx==2
  unemp_m = emp_m_nx==2
  unemp_f = emp_f_nx==2
  nx = sub2ind(model.storage.VFsize, a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx)
  state = IdioState(a, s, d_m, d_f, ret, unemp_m, unemp_f, a_nx, s_nx, d_nx, nx)
end

function bellman_operator2!(VF::Array, policy::PolicyFunctions, model::Model, aggstate::AnAggState; SKIP_HOWARD = false)
  const worktypes = WorkStatus[WW() WN() ; NW() NN()]
  a_grid = model.a_grid
  g, gval, g_worktype_nx = policy.g, policy.gval, policy.g_worktype_nx
  brentguess = model.storage.brentguess
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = size(VF)
  avg_val = zeros(n_a, 2, 2)
  #EVsplines = Array(Interpolations.GriddedInterpolation{Float64,1,
  #  Float64,Interpolations.Gridded{Interpolations.Linear},Tuple{Array{Float64,1}},0}, 2, 2)
  EVsplines = Array(Spline1D, 2, 2)
  amin = a_grid[1]
  amin2 = a_grid[2]
  amax = a_grid[n_a]
  spind = 1:n_a
  const BRUTE_FORCE = false
  const SKIP_GRID = false
  testvals = Inf*ones(n_a)
  beta = model.HB.beta

  enum_nd = Array(Vector{Int}, 2, 2)
  enum_nd[2, 2] = [4]
  enum_nd[2, 1] = [2; 4]
  enum_nd[1, 2] = [3; 4]
  enum_nd[1, 1] = [1; 2; 3; 4]
  currentChoice = ones(length(worktypes)) * amin
  minsofar = zeros(length(worktypes))
  err = 0.
  oldVF = copy(VF)

  newVF = SharedArray(Float64, size(VF), pids = procs() )
  newgval = SharedArray(Float64, size(VF), pids = procs() )
  newg_worktype_nx = SharedArray(Int, size(VF), pids = procs() )

#=
  newVF = zeros(size(VF))
  newgval = zeros(size(VF))
  newg_worktype_nx = zeros(Int, size(VF))
=#
  #@time @sync @parallel for s_nx in 1:n_s
  #  for ret_nx in 1:n_ret, emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d
  @sync @parallel for outnx in 1:prod(size(VF)[[2; 4; 6]] )
    s_nx, emp_m_nx, d_nx = ind2sub(size(VF)[[2;4; 6]], outnx)
    #@time @sync @parallel for d_nx in 1:n_d
    #  for s_nx in 1:n_s, ret_nx in 1:n_ret, emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f
    #EVsplines = Array(Spline1D, 2, 2)
    #avg_val = zeros(n_a, 2, 2)
    for ret_nx in 1:n_ret, emp_f_nx in 1:n_emp_f
      fill!(currentChoice, amin)
      compute_expectation!(s_nx, ret_nx, d_nx, false, false, oldVF, model, slice(avg_val, :, 2, 2), emp_m_nx, emp_f_nx)
      EVsplines[2, 2] = Spline1D(a_grid[spind], avg_val[spind, 2, 2]) # Interpolate
      #EVsplines[2, 2] = interpolate((a_grid,), avg_val[spind, 2, 2], Gridded(Linear()))
      CANWORK_m = (ret_nx == 1 && emp_m_nx == 1)
      CANWORK_f = (ret_nx == 1 && emp_f_nx == 1)
      if CANWORK_m # If male work is an option
        compute_expectation!(s_nx, ret_nx, d_nx, true, false, oldVF, model, slice(avg_val, :, 1, 2), emp_m_nx, emp_f_nx)
        EVsplines[1, 2] = Spline1D(a_grid[spind], avg_val[spind, 1, 2] )
        #EVsplines[1, 2] = interpolate((a_grid,), avg_val[spind, 1, 2], Gridded(Linear()))
      end
      if CANWORK_f # If female work is an option
        compute_expectation!(s_nx, ret_nx, d_nx, false, true, oldVF, model, slice(avg_val, :, 2, 1), emp_m_nx, emp_f_nx)
        EVsplines[2, 1] = Spline1D(a_grid[spind], avg_val[spind, 2, 1])
        #EVsplines[2, 1] = interpolate((a_grid,), avg_val[spind, 2, 1], Gridded(Linear()))
      end
      if CANWORK_m && CANWORK_f # If both work is an option
        compute_expectation!(s_nx, ret_nx, d_nx, true, true, oldVF, model, slice(avg_val, :, 1, 1), emp_m_nx, emp_f_nx)
        EVsplines[1, 1] = Spline1D(a_grid[spind], avg_val[spind, 1, 1] )
        #EVsplines[1, 1] = interpolate((a_grid,), avg_val[spind, 1, 1], Gridded(Linear()))
      end
      # Solve maximization problem for each state
      a_choices = ones(Int, 4)
      for a_nx in 1:n_a
        IS = model.storage.IS_array[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
        state = State(IS, aggstate)
        nx = state.IS.nx
        # Need to solve 4 maximization problems and pick best one
        # If I do it this way, conditioning on work status for each problem,
        # I can avoid non-monotonicity in the decision rules
        fill!(minsofar, Inf)
        for i in (ret_nx == 2 ? [4] : enum_nd[emp_m_nx, emp_f_nx] ) # If retired, cancel work opportunity. Otherwise, select available choices.
          work_nx = ind2sub(size(enum_nd), i)
          WORK_m = work_nx[1]
          WORK_f = work_nx[2]
          OO = worktypes[WORK_m, WORK_f]
          EVspline = EVsplines[WORK_m, WORK_f]
          #EVspline = EVsplines[2, 2]
          obj = pars_bellman_obj(state, EVspline, model, policy, OO)
          #deriv_check = obj( amin2 )  - obj(amin ) # Enforce borrowing constraint by derivative at corner
          feasible_check = obj(amin) # Is this inf? If so, this is not a valid state. We're probably in CRRA
          # Or it is impossible to obtain positive consumption for this hours combination
          # In which case the household will have -Inf utility here, and won't pick it
          #if deriv_check > 0 || isnan(deriv_check)
          if !isfinite(feasible_check)
            currentChoice[i], minsofar[i] = amin, obj(amin)
          else
            max_aprime =  min(a_grid[n_a], 0.99*budget_max_aprime(state, model, OO))
            max_aprime_nx = locate(max_aprime, a_grid)[1] - 1
            if !SKIP_GRID
              fill!(testvals, Inf)
              stuff = a_grid[a_choices[i]:max_aprime_nx]
              testvals[a_choices[i]:max_aprime_nx] = map(obj, stuff)
            end

            #=
            for jk = a_choices[i]:max_aprime_nx
            testvals[jk] = obj(a_grid[jk])
            #end
            =#
            #small_nx = indmin(slice(testvals, a_choices[i]:max_aprime_nx)) + a_choices[i] - 1
            # The maximum should be +/- 1 gridpoint from here.
            small_nx = indmin(testvals)
            #small_nx = indmin(slice(testvals, 1:max_aprime_nx)) + 1 - 1
            # @bp a_nx == 1 && s_nx == 1 && i == 2
            if SKIP_GRID
              lowerguess = a_grid[a_choices[i]]
              upperguess = a_grid[max_aprime_nx]
            else
              lowerguess = a_grid[max(1, small_nx - 1)]
              upperguess = a_grid[min(n_a, small_nx + 1)]
            end
            if BRUTE_FORCE
              (currentChoice[i], minsofar[i]) = lowerguess, testvals[small_nx]
            else
              lowerguess = max(lowerguess, amin)
              upperguess = min(upperguess, amax)
              if !SKIP_GRID && small_nx == 1
                currentChoice[i], minsofar[i] = amin, obj(amin)
              else
                (currentChoice[i], minsofar[i]) = brent(obj, lowerguess,  upperguess)
              end
            end
          end
          a_choices[i] = max(1, locate(currentChoice[i], a_grid)[1] - 2) # Be conservative here
          #a_choices[i] = 1
        end
        #@bp s_nx == ret_nx == emp_m_nx == emp_f_nx == 1 && d_nx == 2
        best_nx = indmin(minsofar)
        # Recover hours info
        currentval = VF[nx]
        newVF[nx] = -minsofar[best_nx]
        newgval[nx] = currentChoice[best_nx]
        newg_worktype_nx[nx] = best_nx
      end
    end
  end
  VF[:] = newVF[:]
  gval[:] = newgval[:]
  g_worktype_nx[:] = newg_worktype_nx[:]
  update_policy!(policy, gval, g_worktype_nx, model, aggstate)
  err = maximum(abs(oldVF - VF))
  err < 1e-8 && return err # Skip the improvement, no need
  SKIP_HOWARD && return err
  # Bellman accelerator step
  n_acc = err < 0.5 ? 15 : 30
  #n_acc = err < 0.5 ? 5 : 10
  Intra_Obj = zeros(VF)
  for k = 1:n_acc
    oldVF[:] = VF[:]
    for s_nx in 1:n_s, ret_nx in 1:n_ret, emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d
      compute_expectation!(s_nx, ret_nx, d_nx, false, false, oldVF, model, slice(avg_val, :, 2, 2), emp_m_nx, emp_f_nx)
      EVsplines[2, 2] = Spline1D(a_grid[spind], avg_val[spind, 2, 2]) # Interpolate
      #EVsplines[2, 2] = interpolate((a_grid,), avg_val[spind, 2, 2], Gridded(Linear()))
      CANWORK_m = (ret_nx == 1 && emp_m_nx == 1)
      CANWORK_f = (ret_nx == 1 && emp_f_nx == 1)
      if CANWORK_m # If male work is an option
        compute_expectation!(s_nx, ret_nx, d_nx, true, false, oldVF, model, slice(avg_val, :, 1, 2), emp_m_nx, emp_f_nx)
        EVsplines[1, 2] = Spline1D(a_grid[spind], avg_val[spind, 1, 2] )
        #EVsplines[1, 2] = interpolate((a_grid,), avg_val[spind, 1, 2], Gridded(Linear()))
      end
      if CANWORK_f # If female work is an option
        compute_expectation!(s_nx, ret_nx, d_nx, false, true, oldVF, model, slice(avg_val, :, 2, 1), emp_m_nx, emp_f_nx)
        EVsplines[2, 1] = Spline1D(a_grid[spind], avg_val[spind, 2, 1])
        #EVsplines[2, 1] = interpolate((a_grid,), avg_val[spind, 2, 1], Gridded(Linear()))
      end
      if CANWORK_m && CANWORK_f # If both work is an option
        compute_expectation!(s_nx, ret_nx, d_nx, true, true, oldVF, model, slice(avg_val, :, 1, 1), emp_m_nx, emp_f_nx)
        EVsplines[1, 1] = Spline1D(a_grid[spind], avg_val[spind, 1, 1] )
        #EVsplines[1, 1] = interpolate((a_grid,), avg_val[spind, 1, 1], Gridded(Linear()))
      end
      for a_nx in 1:n_a
        IS = buildState(a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx, model)
        state = State(IS, aggstate)
        nx = state.IS.nx
        best_nx = g_worktype_nx[nx]
        work_nx = ind2sub(size(enum_nd), best_nx)
        WORK_m = work_nx[1]
        WORK_f = work_nx[2]
        OO = worktypes[WORK_m, WORK_f]
        EVspline = EVsplines[WORK_m, WORK_f]
        #EVspline = EVsplines[2, 2]
        ap = gval[nx]
        if k == 1
          Intra_Obj[nx] = solve_intra_nosolve(state, ap, model, policy)
        end
        EV = EVspline(ap)
        VF[nx] = ( Intra_Obj[nx]  + model.HB.beta * EV )
      end
    end
  end
  return err
end

function update_policy!(policy::PolicyFunctions, gval, g_worktype_nx, model, aggstate)
  const worktypes = WorkStatus[WW() WN() ; NW() NN()]
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = model.storage.VFsize
  for s_nx in 1:n_s, ret_nx in 1:n_ret, emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d
    for a_nx in 1:n_a
      IS = model.storage.IS_array[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
      state = State(IS, aggstate)
      nx = state.IS.nx
      solve_intra(state, gval[nx], model, policy, worktypes[g_worktype_nx[nx]]
      ; update_policy = true)
    end
  end
end


function distribution_operator!(dist::Array, policy::Array{Int64}, policy_mwork::Array{Int},
  policy_fwork::Array{Int}, model::Model, newdist = zeros(dist))
  fill!(newdist, 0.) # Just reusing this temporary array
  IP = model.IP
  const ENDO_UNEMP = true
  sizedist = size(dist)
  bigestate_send_nx::Vector{Int} = model.storage.bigestate_send_nx
  Pi_s::Array{Float64, 5} = IP.Pi_s # Index s, s', ret, ret', d. P(s' | s, ret, ret', d)
  Pi_ret::Array{Float64, 2} = IP.Pi_ret # Index ret, ret'
  Pi_emp_m::Array{Float64, 2} = IP.Pi_emp_m # Index emp', labor
  Pi_emp_f::Array{Float64, 2} = IP.Pi_emp_f
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = size(dist)
  # For all states in the initial distribution
  @inbounds for s_nx in 1:n_s, ret_nx in 1:n_ret, emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d, a_nx in 1:n_a
    # Get the mass of people in this state
    currentmass = dist[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
    currentmass == 0. && continue # Save time
    ap_nx = policy[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
    if ENDO_UNEMP
      work_m_nx = policy_mwork[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
      work_f_nx = policy_fwork[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
    else
      work_m_nx = emp_m_nx
      work_f_nx = emp_f_nx
    end
    # For all possible random future states, calculate the probability of going there
    @inbounds for sp_nx = 1:n_s, retp_nx = 1:n_ret
      prob_sp = Pi_s[s_nx, sp_nx, ret_nx, retp_nx, d_nx]
      (prob_sp == 0.) && continue # Save time
      prob_rp = Pi_ret[ret_nx, retp_nx]
      @inbounds for emp_mp_nx=1:n_emp_m, emp_fp_nx = 1:n_emp_f
        prob_emp_mp = Pi_emp_m[emp_mp_nx, work_m_nx]
        prob_emp_fp = Pi_emp_f[emp_fp_nx, work_f_nx]
        prob = prob_sp * prob_rp * prob_emp_mp * prob_emp_fp
        # If we're being reborn, apply the estate tax
        # Take the mass of people currently in this state, muultiply by the probability of being in the future state
        if retp_nx == 1 && ret_nx == 2
          # Apply estate tax and force employment
          newdist[bigestate_send_nx[ap_nx], sp_nx, retp_nx, 1, 1, d_nx] += currentmass * prob
        else
          newdist[ap_nx, sp_nx, retp_nx, emp_mp_nx, emp_fp_nx, d_nx] += currentmass * prob
        end
      end
    end
  end
  err = maxabsdiff(newdist,dist)
  dist[:] = newdist[:]
  return err
end

function maxabsdiff(x,y)
  size(x) == size(y) || error("Wrong size in maxabsdiff")
  maxsofar = -Inf
  @inbounds for i = 1:length(x)
    maxsofar = max(maxsofar, abs(x[i] - y[i]))
  end
  return maxsofar
end

function produc(k, l, model::Model)
  alpha = model.PT.alpha
  delta = model.PT.delta
  k^(alpha)*l^(1 - alpha) - delta*k
end

function compute_aggregates(dist::Array, model::Model, policy::PolicyFunctions)
  IP = model.IP
  agrid = model.biga_grid
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = size(dist)
  s_vals = IP.s_vals
  gh_m = policy.gh_m
  gh_f = policy.gh_f
  gc = policy.gc
  g_income = policy.g_income
  gval = policy.gval
  hbar = model.UF.hbar
  mu_male = model.IP.mu_male
  mu_female = model.IP.mu_female
  A = 0.
  Lm = 0.
  Lf = 0.
  C = 0.
  Y, Hf, Hm = 0., 0., 0.
  @inbounds for s_nx in 1:n_s, ret_nx in 1:n_ret, emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d, a_nx in 1:n_a
    prob = dist[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
    prob == 0 && continue
    A += prob * agrid[a_nx]
    fhours = gh_f[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
    mhours = gh_m[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
    fhhat = max(0., fhours - hbar)
    mhhat = max(0., mhours - hbar)
    Lm += prob * mu_male * s_vals[s_nx, d_nx] * mhhat
    Lf += prob * mu_female * s_vals[s_nx, d_nx] * fhhat
    C += prob * gc[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
    ap = gval[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
    y = g_income[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
    Y += prob * y
    Hf += prob*fhours*mu_female
    Hm += prob*mhours*mu_male
  end
  adist = zeros(agrid)
  @inbounds for a_nx1 in 1:n_a
    adist[a_nx1] = sum(slice(dist, a_nx1, :, :, :, :, : ))
  end
  acdf = cumsum(adist)
  da = diff(agrid)
  G = 0.
  for a_nx in 1:(n_a - 1)
    Fa = acdf[a_nx]
    d = da[a_nx]
    G += Fa*(1 - Fa)*d
  end

  G = G/(A)

  L = Lm + Lf
  H = Hm + Hf
  Y_prod = produc(A, L, model)
  return Aggregates(A, Lm, Lf, C, L, Hm, Hf, H, Y, Y_prod, G)
end




function locate(x, xgrid) # Assumes monotonic increasing x
  n = length(xgrid)
  n_x = length(x)
  (x[1] >= xgrid[1]  && x[n_x] <= xgrid[n] ) || error("Grid not in bounds $x")

  outIND = zeros(Int64,n_x)
  currentIND = 1
  for k = 1:n_x
    currentval = x[k]
    j = currentIND
    while xgrid[j] < currentval
      j+=1
    end
    currentIND = j ; outIND[k] = j;
  end
  outIND
end


function locateslow2(x, xgrid)
  itp = interpolate((xgrid,), 1:length(xgrid), Gridded(Constant()))
  round(Int, itp[x])
end

function interp1(xgrid,fx,evals)
  # First, find the closest values to evals in xgrid, by index
  # This returns the nearest point to the right of evals. We can interpolate between them
  IND = locate(evals,xgrid)
  n = length(evals)
  retvals = zeros(n)
  for i = 1:n
    cIND = IND[i] # Index of point to the right of evals[i] in xgrid
    cIND == 1 ? (retvals[i] = fx[i] ; continue) : nothing # This point is on the left boundary of xgrid
    t = (evals[i] - xgrid[cIND-1])/(xgrid[cIND] - xgrid[cIND-1])
    retvals[i] = (1-t)*fx[cIND-1] + t*fx[cIND]
  end
  return retvals
end




function interp1d(x_grid, fx, evals)
  EVspline = Spline1D(x_grid, fx, k = 1)
  evaluate(EVspline, evals)
end


function loggrid(min,max,n)
  stretch = 9;
  lin = linspace(0,stretch,n)
  b = (max-min)/(e^stretch-1)
  a = min - b
  out = a + b*exp(lin)
end

#=
%Function TAUCHEN
%
%Purpose:    Finds a Markov chain whose sample paths
%            approximate those of the AR(1) process
%                z(t+1) = amean + rho * z(t) + eps(t+1)
%            where eps are normal with stddev sigmaeps
%
%Format:     {Z, Zprob} = Tauchen(H,amean,rho,sigmaeps,m)
%
%Input:      H      scalar, number of points in z-grid
%            amean       scalar
%            rho     scalar
%            sigmaeps   scalar, std. dev. of epsilons
%            m       max +- std. devs.

usage: (Z, Zprob) = tauchen(n_y,0,rho,sigma,3);
Z = exp(Z) if we meant a log-AR1

=#
function tauchen(H,amean,rho,sigmaeps,m)
  function  cdf_normal(x)
    c = 0.5 * erfc(-x/sqrt(2));
  end
  Z     = zeros(H);
  Zprob = zeros(H,H);

  Z[H] = m * sqrt(sigmaeps^2 / (1 - rho^2));
  Z[1]  = -Z[H];
  zstep = (Z[H] - Z[1]) / (H - 1);

  for i=2:(H-1)
    Z[i]= Z[1] + zstep * (i - 1);
  end

  Z = Z + amean / (1-rho);

  for j = 1:H
    for k = 1:H
      if k == 1
        Zprob[j,k] = cdf_normal((Z[1] - amean - rho * Z[j] + zstep / 2) / sigmaeps);
      elseif k == H
        Zprob[j,k] = 1 - cdf_normal((Z[H] - amean - rho * Z[j] - zstep / 2) / sigmaeps);
      else
        Zprob[j,k] = (cdf_normal((Z[k] - amean - rho * Z[j] + zstep / 2) / sigmaeps) -
        cdf_normal((Z[k] - amean - rho * Z[j] - zstep / 2) / sigmaeps) )
      end
    end
  end

  return (Z, Zprob)
end






function test1(N)
  A = rand(N, 10)
  Pi = rand(10,10)
  EA = zeros(N)
  for i = 1:10
    EA[:] = slice(Pi,i, :)' * A'
  end
end

function test2(N)
  A = rand(N, 10)
  Pi = rand(10,10)
  EA = zeros(N)
  for i = 1:10
    for j = 1:10
      EA[j] = dot(slice(Pi, i, :) , slice(A, i, :))
    end
  end
end

# Compute EV(a' | state). Produce vector over a'
function compute_expectation2(s_nx, ret_nx, emp_nx, d_nx, work::Bool, VF::AbstractArray,
  model::Model, EV = Array(Float64, size(VF, 1)))

  IP = model.IP
  # This uses interpolation and is about half as fast, which is probably negligible
  work_nx = work ? 2 : 1
  Pi_s::Array{Float64, 5} = IP.Pi_s # Index s, s', ret, ret', d. P(s' | s, ret, ret', d)
  Pi_ret::Array{Float64, 2} = IP.Pi_ret # Index ret, ret'
  Pi_emp::Array{Float64, 3} = IP.Pi_emp # Index emp, emp', labor


  n_a, n_s, n_ret, n_emp, n_d = size(VF)

  knots = (model.a_grid, 1:n_s, 1:n_ret, 1:n_emp, 1:n_d)
  inttype = (Gridded(Linear()), NoInterp(), NoInterp(), NoInterp(), NoInterp())
  VF_interp = interpolate(knots, VF, inttype)
  for ap_nx = 1:n_a
    ev = 0.
    for sp_nx = 1:n_s, empp_nx=1:n_emp, retp_nx = 1:n_ret
      prob_sp = Pi_s[s_nx, sp_nx, ret_nx, retp_nx, d_nx]
      if prob_sp == 0
        continue
      else
        prob_rp = Pi_ret[ret_nx, retp_nx]
        prob_empp = Pi_emp[emp_nx, empp_nx, work_nx]
        if retp_nx == 1 && ret_nx == 2
          val = VF_interp[.9*model.a_grid[ap_nx], sp_nx, retp_nx, empp_nx, d_nx]
        else
          val = VF[ap_nx, sp_nx, retp_nx, empp_nx, d_nx]
        end
      end
      prob = prob_sp * prob_rp * prob_empp
      ev += prob * val
    end
    EV[ap_nx] = ev
  end
  return nothing
end
