function test2(model)
  aggstate = model.AST
  sizeVF = model.storage.VFsize
  policy = PolicyFunctions(zeros(Int, sizeVF), zeros(sizeVF), zeros(sizeVF), zeros(sizeVF),
  zeros(sizeVF), zeros(Int, sizeVF), zeros(sizeVF))
  VF = zeros(sizeVF)
  dist = initialdist(model)
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = sizeVF
  a_grid = model.a_grid
  bign = length(model.biga_grid)
  biggval = zeros(Float64, bign, sizeVF[2:end]... )
  sizebig = size(biggval)
  bigpolicy = PolicyFunctions(zeros(Int, sizebig), zeros(sizebig), zeros(sizebig), zeros(sizebig),
  zeros(sizebig), zeros(Int, sizebig), zeros(sizebig))
  bign = length(model.biga_grid)
  biggval = bigpolicy.gval
  @time (VF, policy, dist, aggs) = solve_model(model, VF, dist, policy, bigpolicy)
  (VF, policy, bigpolicy, dist, aggs)
end

function initialdist(model::Model)
  sizeVF = model.storage.VFsize
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = sizeVF
  bign = length(model.biga_grid)
  dist = ones(bign, sizeVF[2:end]... )
  fixdist!(model, dist)
  return dist
end

function fixdist!(model::Model, dist)
  CP = model.CP
  pd1, pd3, ps1d1, ps2d2, ps3d3 = CP.pd1, CP.pd3, CP.ps1d1, CP.ps2d2, CP.ps3d3
  pd2 = 1 - pd1 - pd3
  ps3d2 = 1 - ps3d3
  ps2d1 = 1 - ps1d1
  ps2d3 = 1 - ps3d3
  dweights = [pd1*ps1d1 ;pd1*ps2d1; pd2*ps2d2; pd2*ps3d3; pd3*ps2d3; pd3*ps3d3]
  n_d = size(dist, 6)
  for d_nx in 1:n_d
    sumd = dweights[d_nx]
    dist[:, :, :, :, :, d_nx] = dist[:, :, :, :, :, d_nx] * sumd / sum(dist[:, :, :, :, :, d_nx])
  end
  dist[:] = dist[:]/sum(dist)
  return nothing
end


SteadyState(model::Model) = SteadyState(solveGE(model)...)
function solveGE(model)
  sizeVF = model.storage.VFsize
  policy = PolicyFunctions(zeros(Int, sizeVF), zeros(sizeVF), zeros(sizeVF), zeros(sizeVF),
  zeros(sizeVF), zeros(Int, sizeVF), zeros(sizeVF))
  VF = zeros(sizeVF)
  dist = initialdist(model)
  solveGE(model, VF, policy, dist)
end

function r2wage(r::Number, model::Model)
  alpha = model.PT.alpha
  delta = model.PT.delta
  KoL = ( (r + delta)/alpha)^(1/(alpha-1))
  w = (1-alpha)*KoL^(alpha)
end

function rstar_GE(A, L, model::Model)
  alpha = model.PT.alpha
  delta = model.PT.delta
  alpha * (A/L)^(alpha-1) - delta
end

function rstar_GE(aggs::Aggregates, model::Model)
  A = aggs.A
  L = aggs.L
  rstar_GE(A, L, model::Model)
end

AggState{UF <: UtilityFunction}(r, w, model::Model{UF, AggState}) = AggState(r, w)
AggState{UF <: UtilityFunction}(r, w, model::Model{UF, AggStateSticky}) = AggStateSticky(r, w, 0.)


function solveGE(model, VF, policy, dist)
  aggstate = model.AST
  r = aggstate.r
  w = r2wage(r, model)
  aggstate = AggState(r, w, model)
  model.AST = aggstate
  sizeVF = model.storage.VFsize
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = sizeVF
  a_grid = model.a_grid
  bign = length(model.biga_grid)
  biggval = zeros(Float64, bign, sizeVF[2:end]... )
  sizebig = size(biggval)
  bigpolicy = PolicyFunctions(zeros(Int, sizebig), zeros(sizebig), zeros(sizebig), zeros(sizebig),
  zeros(sizebig), zeros(Int, sizebig), zeros(sizebig))


  bign = length(model.biga_grid)
  biggval = bigpolicy.gval
  @time (VF, policy, dist, aggs) = solve_model(model, VF, dist, policy, bigpolicy)
  rstar = rstar_GE(aggs.A, aggs.L, model)

  iter = 0
  while abs(r - rstar) > 1e-5
  #while false
    r = 0.5*r + 0.5*rstar
    w = r2wage(r, model)
    aggstate = AggState(r, w, model)
    model.AST = aggstate
    @time (VF, policy, dist, aggs) = solve_model(model, VF, dist, policy, bigpolicy)
    rstar = rstar_GE(aggs.A, aggs.L, model)
    @show abs(r - rstar)
    iter += 1
    if iter > 50
      @show "Giving up on GE"
      break
    end
  end
  aggstate = AggState(r, w, model)
  model.AST = aggstate
  return (VF, policy, bigpolicy, dist, aggs, r)
end

function makebigpolicy!(bigpolicy, policy, model)
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = model.storage.VFsize
  a_grid = model.a_grid
  bign = length(model.biga_grid)
  biggval = bigpolicy.gval
  biggval_nx = bigpolicy.g
  biga_grid = model.biga_grid
  biggc = bigpolicy.gc
  biggh_m = bigpolicy.gh_m
  biggh_f = bigpolicy.gh_f
  bigg_income = bigpolicy.g_income
  for s_nx in 1:n_s, ret_nx in 1:n_ret, emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d
    biggval_interp = interpolate((a_grid,), slice(policy.gval, :, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx),
    Gridded(Linear()) )
    biggval[:, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx] = biggval_interp[biga_grid]
    biggval_nx[:, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx] = locateslow2(slice(biggval, :, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx ), biga_grid)
    biggc_interp = interpolate((a_grid,), slice(policy.gc, :, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx),
    Gridded(Linear()) )
    biggc[:, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx] = biggc_interp[biga_grid]
    biggh_m_interp = interpolate((a_grid,), slice(policy.gh_m, :, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx),
    Gridded(Linear()) )
    biggh_m[:, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx] = biggh_m_interp[biga_grid]
    biggh_f_interp = interpolate((a_grid,), slice(policy.gh_f, :, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx),
    Gridded(Linear()) )
    biggh_f[:, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx] = biggh_f_interp[biga_grid]
    bigg_income_interp = interpolate((a_grid,), slice(policy.g_income, :, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx),
    Gridded(Linear()) )
    bigg_income[:, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx] = bigg_income_interp[biga_grid]
  end
  #=
  tempbiggval_nx = SharedArray(Int, size(biggval_nx))
  @sync @parallel for nx in 1:prod(size(biggc)[2:6])
    (s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx) = ind2sub(size(biggc)[2:6], nx)
    tempbiggval_nx[:, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx] = locateslow(slice(biggval, :, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx ), biga_grid)
  end
  biggval_nx[:] = tempbiggval_nx
  =#
end

function makebigVF!(bigVF, VF, model)
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = model.storage.VFsize
  a_grid = model.a_grid
  bign = length(model.biga_grid)
  biga_grid = model.biga_grid
  for s_nx in 1:n_s, ret_nx in 1:n_ret, emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d
    bigVF_interp = interpolate((a_grid,), slice(VF, :, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx),
    Gridded(Linear()) )
    bigVF[:, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx] = bigVF_interp[biga_grid]
  end
end




function solve_model(model, VF, dist, policy, bigpolicy)
  aggstate = model.AST
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = model.storage.VFsize
  a_grid = model.a_grid
  sizeVF = model.storage.VFsize
  bign = length(model.biga_grid)
  biggval = zeros(Float64, bign, sizeVF[2:end]... )
  sizebig = size(biggval)

  err = 1.
  iter = 0
  while err > 1e-6
    #for k = 1:20
    @show err = bellman_operator2!(VF, policy, model, aggstate)
    iter += 1
    if
      iter == 55
      @show "Giving up on Bellman equation"
      break
    end
  end

  makebigpolicy!(bigpolicy, policy, model)

  policy_mwork = 2 - (bigpolicy.gh_m .> 0)
  policy_fwork = 2 - (bigpolicy.gh_f .> 0)
  err = 1.
  iter = 0
  newdist = zeros(dist)
  while err > 1e-8
    err = distribution_operator!(dist, bigpolicy.g, policy_mwork, policy_fwork, model, newdist)
    iter += 1
    if iter > 1000
      @show "Giving up on distribution"
      break
    end
  end
  aggs = compute_aggregates(dist, model, bigpolicy)
  (VF, policy, dist, aggs)
end

 probind = [4; 5; 6; 7; 8] # Which ones are probability?
function ControlParameters(vec::Vector)
  vec = copy(vec)
  ind = 1:length(vec)
  vec[probind] = invlogit(vec[probind])
  notind = setdiff(ind, probind)
  vec[notind] = exp(vec[notind])
  ControlParameters(vec...)
end

# Convert initial guess of parameters into the right format for the solver
function invControlParameters(args...)
  ind = 1:length(args)
  vec = Float64[a for a in args]
  vec[probind] = logit(vec[probind])
  notind = setdiff(ind, probind)
  vec[notind] = log(vec[notind])
  return vec
end
function ControlParameters(dm1, dm2, dm3, pd1, pd3, ps1d1, ps2d2, ps3d3, alpha0)
  dm = [dm1; dm1; dm2; dm2; dm3; dm3]
  sL = 1/2.
  sH = 1.
  sHH = 2.
  s = [sL; sH; sH; sHH; sH; sHH]
  alpha1 = 1.1
  sigmal = 0.88
  hbar = 0.05
  cbar = 0.4
  mu_male = 1/3.2
  mu_female = 0.77 * 0.854/3.2
  CP = ControlParameters(dm, pd1, pd3, ps1d1, ps2d2, ps3d3, alpha0, alpha1, s, cbar, sigmal, hbar, mu_male, mu_female)
end

invlogit(x) = exp(x) ./ (1 + exp(x))
logit(x) = log(x./(1-x))

function calibrate(model::Model)
  function werr(targets::Array)
    CP = ControlParameters(targets)
    rstar = fakerstar[1]
    @show CP
    model = makeModel(CP, rstar = rstar)
    (VF2, policy, bigpolicy, dist2, aggs, rstar) = solveGE(model, VF, policy, dist);
    VF[:] = VF2[:] # Transmit this VF outside so we can reuse it in future
    dist[:] = dist2[:]
    fakerstar[1] = rstar
    @show errs = check_moments(dist, model, bigpolicy);
    return sum(errs.^2)
  end
  fakerstar = zeros(1)
  dm1 = 0.15
  dm2 = 0.03
  dm3 = 0.008
  pd1 = 0.12
  pd3 = 0.5
  ps1d1 = 0.7
  ps2d2 = 0.6
  ps3d3 = 0.2
  alpha0 = 10.
  targets = invControlParameters(dm1, dm2, dm3, pd1, pd3, ps1d1, ps2d2, ps3d3, alpha0)
  CP = ControlParameters(targets)
  model = makeModel(CP)
  # init everything
  initial_step = targets + .2
  (VF, policy, bigpolicy, dist, aggs, rstar) = solveGE(model)
  fakerstar[1] = rstar
  #res = optimize(werr, targets, iterations = 500, initial_step = initial_step)
  res = Optim.nelder_mead(werr, targets, initial_step = initial_step, iterations = 500)
  t = time()
  save("$t.jld", "res", res)
  return res
  #res = Optim.levenberg_marquardt(werr, jerr, targets0, maxIter = 20)
end
