## Identify bins (quintiles, s type, etc)
# Pick the representative (median) person in the bins

# For each representative
# We know policy
# Get their n-step ahead distribution for the expectation
# This uses the distribution operator. Policy functions fixed

# Compute V0(bin, lambda)
# This is U(lambda * c(state), l(state)) + beta * EU (lambda ... states) + ...
# So we need to compute U by state, given policy. This is easy
# Loop over all the states. If they have positive probability in pi, compute utility. Repeat.

function V0(bin::Array, lambda::Number, model::Model, policy::PolicyFunctions)
  bin = copy(bin)
  V = 0.
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = size(bin)
  beta = model.HB.beta
  gc = policy.gc
  gh_m = policy.gh_m
  gh_f = policy.gh_f
  g = policy.g
  policy_mwork = 2 - (gh_m .> 0)
  policy_fwork = 2 - (gh_f .> 0)
  IS_array = Array(IdioState, size(bin))
  @inbounds for s_nx in 1:n_s, ret_nx in 1:n_ret, emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d, a_nx in 1:n_a
    is = buildState_big(a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx, model)
    IS_array[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx] = is
  end

  # which d type are we?
  # The bin is a unit mass at 1 point
  # For t in 1:T
  for t in 1:200
    discount = beta^(t - 1)
    # For every element in the state space
    Vt = 0.
    @inbounds for (nx, currentmass) in enumerate(bin)
      # Is there any mass here?
      currentmass == 0. && continue
      # Get the consumption and labor policy for this point
      c = gc[nx]
      h_m = gh_m[nx]
      h_f = gh_f[nx]
      state = State(IS_array[nx], model.AST)
      u = U(lambda*c, h_m, h_f, state, model)
      # Compute the utility given lambda and muultiply with the mass
      # Accumulate
      Vt += u * currentmass
    end
    # Accumulate
    V += discount * Vt
    # Update the bin distribution
    distribution_operator!(bin, g, policy_mwork, policy_fwork, model)
  end
  return V
end

function lambda_bin(bin_nx, VF, model, exp_size, steady)
  # Compute V(bin)
  # This is just the value function evaluated at t = 2 including the transfer (a + x(a))
  policy = steady.bigpolicy
  size(VF) == size(policy.gc) || error("Error")
  MPCpolicy_nx = getMPCpolicy_nx(exp_size, steady, model)

 ### FIX THIS
 ### MPCpolicy_nx is sending everyone to type 1. You need the full index

  ap_nx = MPCpolicy_nx[bin_nx...]
  x_nx = (ap_nx, bin_nx[2:end]...)
  Vbin = VF[x_nx...]
  bin = zeros(VF)
  @show bin_nx
  bin[bin_nx...] = 1
  # Solve for lambda in V(bin) = V0(bin, lambda)
  flam = lambda -> V0(bin, lambda, model, policy) - Vbin
  #Roots.fzero(flam, 0.5, 2.)
  myNewton2(flam, 1., 0., 5.)
end

function getlambdas(bin_nxs, VF, model, steady, exp_size)
  policy = steady.bigpolicy
  bigVF = zeros(policy.gc)
  makebigVF!(bigVF, VF, model)
  lambda = zeros(size(bin_nxs))
  _lambda_bin(bin_nx) = lambda_bin(bin_nx, bigVF, model, exp_size, steady)
  lambda[:] = pmap(_lambda_bin, bin_nxs)
  #=
  for (nx, bin_nx) in enumerate(bin_nxs)
    lambda[nx] = lambda_bin(bin_nx, bigVF, model, exp_size, steady)
  end
  =#
  return lambda
end

function makebins(steady::SteadyState, model::Model, exp_size::Number)
  VF2 = GEexperiment(exp_size, steady, model; retVF2 = true)
  dist = steady.dist
  agrid = model.biga_grid
  APDF = Float64[sum(dist[i, :]) for i in 1:length(agrid)]
  ACDF = cumsum(APDF)
  aquint = zeros(6)
  quint = [0.1; 0.3; 0.5; 0.7; 0.9; 1.]
  # Fix this so you use the heterogeneity in other types
  # Right now >20% of people can be at 0 assets
  ###### Asset quintile setup
  aquint = Int64[quantileCDF(ACDF, quint[i] ) for i in 1:length(aquint) ]
  aquint = aquint[1:5]
  sind = 1:5
  dind = 1:size(dist, 6)
  mynx = Tuple{Int, Int, Int, Int, Int, Int}[s_nx == 5 ? (a_nx, 1, 2, 1, 1, d_nx) :
  (a_nx, s_nx, 1, 1, 1, d_nx) for a_nx in aquint, s_nx in sind, d_nx in dind]
  bin_nxs = mynx
  getlambdas(bin_nxs, VF2, model, steady, exp_size)
end

# Find lambda s.t. E V0(lambda) = EV1
function singlelambda(steady::SteadyState, model::Model, exp_size::Number; rvec = zeros(T) )
  if rvec != zeros(T)
    VF2 = experiment(rvec, 0.98, steady, model, PE = false, retVF2 = true)
  else
    (VF2, aggss) = GEexperiment(exp_size, steady, model; retVF2 = true)
  end
  policy = steady.bigpolicy
  dist = steady.dist
  dist2 = MPC(exp_size, steady, model)
  bigVF2 = zeros(dist2)
  makebigVF!(bigVF2, VF2, model)
  bigVF2[bigVF2 .== -Inf] = 0.
  EVF2 = sum(bigVF2.*dist2)
  function flam(lambda)
    V0(dist, lambda, model, policy) - EVF2
  end
  myNewton2(flam, 1., 0., 5.)
end
