
function incometax(y::Number, model::Model)
  t = model.IT
  a0, a1, a2, a3 = t.a0, t.a1, t.a2, t.a3
  return (y > 0. ?
  a0*(y-(y^(-a1)+a2)^(-1/a1))+a3*y :
  zero(y) )
end
function incometax2(y::Number, model::Model)
  itp = model.storage.itp
  itp[y]
end

function buildincometax2(model::Model)
  agrid = model.a_grid
  fx = Float64[incometax(y, model) for y in agrid]
  itp = interpolate((agrid,), fx, Gridded(Linear()) )
end


function incometaxprime(y::Number, model::Model)
  t = model.IT
  a0, a1, a2, a3 = t.a0, t.a1, t.a2, t.a3
  tp=a0 * ( 1-(y^(-a1)+a2)^( -(1+a1)/a1) *y^(-(1+a1)) ) + a3
end

function estatetax(y::Number, model::Model)
  t = model.ET
  tauE, zlower = t.tauE, t.zlower
  taua=max(zero(y), tauE*(y-zlower))
end

function budget_max_aprime(state::State, model::Model, OO::WorkStatus)
  mhours = mhours_fun(OO, model)
  fhours = typeof(OO) <: Union{NW, WW} ? model.UF.hoursmax : 0. # Household female is working positive hours, could be max. Otherwise 0.
  budget_max_aprime(state, model, mhours, fhours)
end

function budget_max_aprime(state::State, model::Model, mhours, fhours)
  c = 0.
  # Since c and a' are symmetric in the budget constraint, we can just swap them
  max_aprime = budget_c_given_h(mhours, fhours, state, c, model, true)
end

# The min female hours that produces positive consumption given the a' choice
function budget_hours_minconsump_fun(mhours::Number, state::State, ap::Number, model::Model)
  # Because of the taxes this requires the nonlinear equation solver
  # Solve for the hours that generate 0 consumption
  # ?? this may not exist. capped at 0
  f = pars_budget_c_given_h(state, ap, model, mhours, false)
  hbar = model.UF.hbar
  hoursmax = model.UF.hoursmax
  if f(hbar) >= 0.
    return hbar
  else
    nx = state.IS.nx
    if mhours == 0.
      #  fhours_guess = model.storage.budget_hours_minconsump_guess_NW[nx]
      fhours_guess = 0.9hbar + 0.1hoursmax
      fhours = myNewton2(f, fhours_guess, hbar, hoursmax)
      #model.storage.budget_hours_minconsump_guess_NW[nx] = fhours
    else
      #fhours_guess = model.storage.budget_hours_minconsump_guess_WW[nx]
      fhours_guess = 0.9hbar + 0.1hoursmax
      fhours = myNewton2(f, fhours_guess, hbar, hoursmax)
      #model.storage.budget_hours_minconsump_guess_WW[nx] = fhours
      return fhours
    end
  end
end


# This is the "wage"


function DcDh(mhours::Number, fhours::Number,
  state::State, ap::Number, model::Model) # Female H automatically
  basic = false # Since this is for FOC, IGNORE the basic income
  f = pars_budget_c_given_h(state, ap, model, mhours, basic)

  w, ar  = ForwardDiff.derivative(f, fhours, ForwardDiff.AllResults)
  c = ForwardDiff.value(ar)
  #=
  inc = aftertax_income_fun(mhours::Number, fhours::Number,
   state::State, ap::Number, model::Model, basic)
  w2 = state.AS.w * state.IS.s * model.IP.mu_female *(1 - incometaxprime(inc, model))
  c = f(fhours)
  =#
  return (c, w)
  #return (c, w2)
end

# We set basic = false only when working on the FOC
function budget_c_given_h(mhours::Number, fhours::Number,
  state::State, ap::Number, model::Model, basic::Bool = true)
  IS = state.IS
  cbar = model.IP.cbar
  a = IS.a
  net_saving = ap - a
  aftertax_income = aftertax_income_fun(mhours, fhours, state, ap, model, basic)
  if basic
    omega_ays = max(0., cbar - aftertax_income - a)
    c = aftertax_income + omega_ays - net_saving
  else
    c = aftertax_income - net_saving
  end
  return c
end
Base.call(f::pars_budget_c_given_h, fhours) = budget_c_given_h(
f.mhours, fhours, f.state, f.ap, f.model, f.basic )

function aftertax_income_fun(mhours::Number, fhours::Number,
  state::State, ap::Number, model::Model, basic::Bool = true)
  AS = state.AS
  IS = state.IS
  r = AS.r
  w = AS.w
  a = IS.a
  hbar = model.UF.hbar
  mu_male = model.IP.mu_male
  mu_female = model.IP.mu_female
  s = IS.s
  ret = IS.ret # Bool, true if retired
  #unemp = mhours == 0. && !ret
  omega = model.HB.omega
  cbar = model.IP.cbar
  fhhat = max(zero(fhours), fhours - hbar)
  mhhat = max(zero(mhours), mhours - hbar)
  gross_income = w*s*(mu_male*mhhat + mu_female*fhhat) + r*a + ret*omega #+ 0.3*unemp*w*1*mu_male #+ 0.1*omega # Female hours less productive (assumption)
  taxable_income = w*s*(mu_male*mhhat + mu_female*fhhat) + ret*omega
  aftertax_income = gross_income - incometax2(taxable_income, model)
end


############################ Special functions just for computing statistics, not the model
function aftertax_income_fun_basic(mhours::Number, fhours::Number,
  state::State, ap::Number, model::Model, basic::Bool = true)
  IS = state.IS
  cbar = model.IP.cbar
  a = IS.a
  aftertax_income = aftertax_income_fun(mhours, fhours, state, ap, model, basic)
  omega_ays = max(0., cbar - aftertax_income - a)
  return aftertax_income + omega_ays
end
# This function excludes transfers but includes taxes. Total earnings from work and assets
function earnings_fun(mhours::Number, fhours::Number,
  state::State, ap::Number, model::Model)
  AS = state.AS
  IS = state.IS
  r = AS.r
  w = AS.w
  a = IS.a
  hbar = model.UF.hbar
  mu_male = model.IP.mu_male
  mu_female = model.IP.mu_female
  s = IS.s
  ret = IS.ret # Bool, true if retired
  unemp = IS.unemp_f || IS.unemp_m
  omega = model.HB.omega
  fhhat = max(zero(fhours), fhours - hbar)
  mhhat = max(zero(mhours), mhours - hbar)
  gross_income = w*s*(mu_male*mhhat + mu_female*fhhat) + r*a
  earnings = gross_income - incometax2(gross_income, model)
end
function beforetax_inc_fun(mhours::Number, fhours::Number,
  state::State, ap::Number, model::Model)
  AS = state.AS
  IS = state.IS
  r = AS.r
  w = AS.w
  a = IS.a
  hbar = model.UF.hbar
  mu_male = model.IP.mu_male
  mu_female = model.IP.mu_female
  s = IS.s
  ret = IS.ret # Bool, true if retired
  unemp = IS.unemp_f || IS.unemp_m
  omega = model.HB.omega
  fhhat = max(zero(fhours), fhours - hbar)
  mhhat = max(zero(mhours), mhours - hbar)
  gross_income = w*s*(mu_male*mhhat + mu_female*fhhat) + r*a
  earnings = gross_income
end


function aftertax_income_fun(mhours::AbstractArray, fhours::AbstractArray,
  state::AbstractArray, ap::AbstractArray, model::Model, basic::Bool = true)
  out = zeros(size(mhours))
  for i in 1:length(mhours)
    is = state[i]
    state_actual = State(is, model.AST)
    out[i] = aftertax_income_fun(mhours[i], fhours[i], state_actual, ap[i], model, true)
  end
  out
end

function test_income(policy::PolicyFunctions, model::Model)
  gh_m, gh_f, gval, gc = policy.gh_m, policy.gh_f, policy.gval, policy.gc
  IS_array = model.storage.IS_array
  ginc = aftertax_income_fun(gh_m, gh_f, IS_array, gval, model)
  a_grid = model.a_grid
  ginc - gc - gval
  a = gc + gval - ginc
end




function buildState_big(a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx, model::Model{CRRA_Family})
  a = model.biga_grid[a_nx]
  s = model.IP.s_vals[s_nx, d_nx]
  UF = model.UF
  d_m = UF.d_m[d_nx]
  d_f = UF.d_f[d_nx]
  ret = ret_nx==2
  unemp_m = emp_m_nx==2
  unemp_f = emp_f_nx==2
  msize = (length(model.biga_grid), model.storage.VFsize[2:end]... )
  nx = sub2ind(msize, a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx)
  state = IdioState(a, s, d_m, d_f, ret, unemp_m, unemp_f, a_nx, s_nx, d_nx, nx)
end

function buildState_big(a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx, model::Model{CRRA})
  a = model.biga_grid[a_nx]
  s = model.IP.s_vals[s_nx, d_nx]
  UF = model.UF
  d_m = Inf
  d_f = UF.d_f[d_nx]
  ret = ret_nx==2
  unemp_m = emp_m_nx==2
  unemp_f = emp_f_nx==2
  msize = (length(model.biga_grid), model.storage.VFsize[2:end]... )
  nx = sub2ind(msize, a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx)
  state = IdioState(a, s, d_m, d_f, ret, unemp_m, unemp_f, a_nx, s_nx, d_nx, nx)
end



function check_moments(dist::Array, model::Model, policy::PolicyFunctions)
  #@show CH
  # These stats should be for everyone
  (CH, wealthquint, incquint, demp, noassets, fquint, Y, medy, avg_H, medINC) = compute_aggregates_extra(dist, model, policy; youngonly = false, maketables = true)
  println("Wealth quintiles are $wealthquint")
  println("Earnings quintiles are $incquint")
  println("Average income is $Y")
  println("Median income is $medy")
  println("Median beforetax earnings is $medINC")
  # These stats should be for young only
  (CH, wealthquint, incquint, demp, noassets, fquint, Y, medy, avg_H, medINC) = compute_aggregates_extra(dist, model, policy; youngonly = true)
  println("Male/female participation rates by wealth quintile is \n $demp")
  println("Intensive margin stats, .05,.1, .25, .5, .75, .9, max, are $fquint")
  println("Average female hours conditional on working is $avg_H")
  CH_target = 3.
  avg_H_target = 0.854

  #wealthquint_target = [0.21; 1.21; 1.93; 14.68; 81.97]
  #incquint_target = [0.1; 3.74; 14.59; 15.99; 65.68]
  dempt_target = [68.8 56.1 ;
  88.7 63.8;
  89.9 69.5;
  91.7 60.;
  93.4 53.7]/100
  a_target = 0.2
  a2_target = model.IP.cbar/medy

  err = [avg_H/ avg_H_target;
  demp[[1; 5; 6; 10]]./dempt_target[[1; 5; 6; 10]] ;
  a2_target/a_target] - 1
end

quantileCDF(A::AbstractArray, p::Number) = findfirst(x -> x >= p, A)
function quantileP(fun::AbstractArray, dist::AbstractArray, p::Number)
  sortedfun = sortperm(fun[:])
  funCDF = cumsum(dist[sortedfun])
  medy = fun[sortedfun[quantileCDF(funCDF, p)]]
end
function quantileP{NU <:Number}(fun::AbstractArray, dist::AbstractArray, p::Vector{NU})
  sortedfun = sortperm(fun[:])
  funCDF = cumsum(dist[sortedfun])
  medy = Float64[fun[sortedfun[quantileCDF(funCDF, px)]] for px in p]
end

function compute_aggregates_extra(dist::Array, model::Model, policy::PolicyFunctions; youngonly = false, maketables = false)
  IP = model.IP
  agrid = model.biga_grid
  n_a, n_s, n_ret, n_emp_m, n_emp_f, n_d = size(dist)
  s_vals = IP.s_vals
  gh_m = policy.gh_m
  gh_f = policy.gh_f
  gc = policy.gc
  gval = policy.gval
  hbar = model.UF.hbar
  mu_male = model.IP.mu_male
  mu_female = model.IP.mu_female
  IS_array = Array(IdioState, size(dist))
  yarray = zeros(dist)
  earnings_array = zeros(dist)
  A, Lm, Lf, C, Y, H, Hf = 0., 0., 0., 0., 0., 0., 0.
  Hm = 0.

  if youngonly
    dist2 = copy(dist)
    dist2[:, :, 2, :, :, :] = 0.
    dist = dist2/sum(dist2)
  end

  @inbounds for s_nx in 1:n_s, ret_nx in 1:n_ret, emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d, a_nx in 1:n_a
    prob = dist[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
    #prob == 0 && continue
    A += prob * agrid[a_nx]
    is = buildState_big(a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx, model)
    IS_array[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx] = is
    state = State(is, model.AST)
    fhours = gh_f[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
    mhours = gh_m[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
    ap = gval[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
    y = aftertax_income_fun_basic(mhours, fhours, state, ap, model)
    yarray[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx] = y
    earnings_array[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx] = earnings_fun(mhours, fhours, state, ap, model)
    Y += prob * y
    fhhat = max(0., fhours - hbar)
    mhhat = max(0., mhours - hbar)
    H += prob * (mu_female*fhours + mu_male*mhours) # ??? not sure how to properly define this
    Hf += prob * (mu_female*fhours) # ??? not sure how to properly define this
    Hm += prob * (mu_male*mhours) # ??? not sure how to properly define this
    Lm += prob * mu_male * s_vals[s_nx, d_nx] * mhhat
    Lf += prob * mu_female * s_vals[s_nx, d_nx] * fhhat
    C += prob * gc[a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx]
  end
  L = Lm + Lf


  # 2nd order moments
  stdHf = sum( ((gh_f) - Hf).^2 .* dist)^(1/2)
  stdC = sum( (gc - C).^2 .* dist)^(1/2)
  CH = (stdC/C) / (stdHf/Hf)
  APDF = Float64[sum(dist[i, :]) for i in 1:length(agrid)]
  ACDF = cumsum(APDF)
  aquint = zeros(6)
  quint = [0.; 0.2; 0.4; 0.6; 0.8; 1.]


  # Fix this so you use the heterogeneity in other types
  # Right now >20% of people can be at 0 assets
  ###### Asset quintile setup
  aquint = Int64[quantileCDF(ACDF, quint[i] ) for i in 1:length(aquint) ]
  aquint[1] = 0
  aquint[6] = length(ACDF)
  quintIND = UnitRange{Int64}[(aquint[i] + 1):aquint[i+1]  for i in 1:5]
  #@show quintIND
  ss = [sum(dist[ind, : ] ) for ind in quintIND] # make sure there are really 20% in the quintiles
  #println("Percent in each wealth quintile is $ss")
  ####### Income quintile setup
  # First sort the state space by income
  iquint = zeros(6)
  sortedy = sortperm(earnings_array[:])
  # Compute the CDF of the income sorted distribution and get quintiles
  ICDF = cumsum(dist[sortedy])
  iquint = Int64[quantileCDF(ICDF, quint[i] ) for i in 1:length(aquint) ]
  iquint[1] = 0
  iquint[6] = length(ICDF)
  iquintIND = UnitRange{Int64}[(iquint[i] + 1):iquint[i+1]  for i in 1:5]
  # "invert" and obtain the indices in the original space of each quintile
  iquintIND = [sortedy[ind] for ind in iquintIND]


  ###### Compute the wealth and income quintiles
  wealthquint = 100*Float64[sum(agrid[ind] .* dist[ind, :] ) for ind in quintIND]/A # Wealth qunitiles.
  incquint = 100*Float64[sum(earnings_array[ind] .* dist[ind] ) for ind in iquintIND]/ sum(earnings_array.*dist) # Income qunitiles.


  ##### What's median earnings? Get the index of the median in the earnings distribution and get median
  medINC = earnings_array[sortedy[quantileCDF(ICDF, 0.5) ]]
  #println("Median earnings is $medINC")


  ###### Logical arrays for male and female labor force participation
  mworkingIND = gh_m .> 0.
  fworkingIND = gh_f .> 0.

  ##### Relative coefficient of variations for consumption and hours, working females only
  C2 = sum(gc[fworkingIND] .* dist[fworkingIND])
  Hf2 = sum(gh_f[fworkingIND] .* dist[fworkingIND])
  stdC2 = sum( (gc[fworkingIND] - C2).^2 .* dist[fworkingIND])^(1/2)
  stdHf2 = sum( (gh_f[fworkingIND] - Hf2).^2 .* dist[fworkingIND])^(1/2)
  CH2 = (stdC2/C2) / (stdHf2/Hf2)

  # Employment rate defined as percent working who could work
  workingIND = mworkingIND | fworkingIND
  # This is great, really flexible syntax
  couldworkIND = Bool[ret_nx == 1 && (emp_m_nx == 1 || emp_f_nx == 1)
  for a_nx in 1:n_a, s_nx in 1:n_s, ret_nx in 1:n_ret,
    emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d]

  u1 = sum(dist[workingIND]) / sum(dist[couldworkIND])
  notretiredIND = ones(Bool, size(dist))
  notretiredIND[:, :, 2, :, :, :] = false
  u2 = sum(dist[workingIND]) / sum(dist[notretiredIND])
  # Employment rate defined as percent working who aren't retired

  # Percent who could work who aren't retired
  u3 = sum(dist[couldworkIND]) / sum(dist[notretiredIND])

  # Share of households with no assets
  noassetsIND = Bool[ agrid[a_nx] < 0.05
  for a_nx in 1:n_a, s_nx in 1:n_s, ret_nx in 1:n_ret,
  emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d]
  noassets = sum(dist[noassetsIND])


  # Labor force participation by gender and wealth quintile, not retired

  # Complete logical arrays representing the quintiles
  quintIND3 = [Bool[a_nx in ind
  for a_nx in 1:n_a, s_nx in 1:n_s, ret_nx in 1:n_ret,
  emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d]
  for ind in quintIND]

  ######## Labor force participation by gender and wealth quintile

  mworkquint = 100*Float64[sum(dist[ind & mworkingIND])/sum(dist[ind]) for ind in quintIND3]
  fworkquint = 100*Float64[sum(dist[ind & fworkingIND])/sum(dist[ind]) for ind in quintIND3]


  ####### Logical arrays for the d types
  dIND = [Bool[d_nx in ind
  for a_nx in 1:n_a, s_nx in 1:n_s, ret_nx in 1:n_ret,
    emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d]
    for ind in 1:n_d ]


  ######## Male and female employment by d type, not retired

  demp = Float64[sum(dist[ind & workIND & notretiredIND]) / sum(dist[ind & notretiredIND]) for ind in quintIND3, workIND in (mworkingIND, fworkingIND) ]

  ###### Average female hours conditional on female working
  fH = sum(gh_f[fworkingIND] .* dist[fworkingIND] ) / sum(dist[fworkingIND])
  #  println("Average female hours conditional on female working is $fH")

  ##### Generate a CDF for the distribution of female hours
  ##### Condition on working
  # Conditioned variables
  gh_f_CV = gh_f[fworkingIND]
  dist_CV = dist[fworkingIND] / sum(dist[fworkingIND])
  sortedfH = sortperm(gh_f_CV)
  fCDF = cumsum(dist_CV[sortedfH]) # First sort and condition the distribution, then CDF it
  # The minimum is incorrect because I interpolate the policy functions in the distribution step
  # So you'll have
  myquint = [.05; .1; .25; .5; .75; .9]
  fquint = Float64[gh_f_CV[sortedfH[quantileCDF(fCDF, m)]] for m in myquint]
  fquint = [fquint; maximum(gh_f_CV)]

  #### Average hours
  avg_H = sum(gh_f_CV.*dist_CV)


  ##### Get median income
  sortedinc = sortperm(yarray[:])
  yCDF = cumsum(dist[sortedinc])
  medy = yarray[sortedinc[quantileCDF(yCDF, 0.5)]]


  if maketables
    quint = [0.; 0.2; 0.4; 0.6; 0.8; 1.; .9; .95; .99]

    # Fix this so you use the heterogeneity in other types
    # Right now >20% of people can be at 0 assets
    ###### Asset quintile setup
    aquint = Int64[quantileCDF(ACDF, quint[i] ) for i in 1:9 ]
    aquint[1] = 0
    aquint[6] = length(ACDF)
    quintIND = UnitRange{Int64}[(aquint[i] + 1):aquint[i+1]  for i in 1:8]
    quintIND[6] = (aquint[7]+1):aquint[8]
    quintIND[7] = (aquint[8]+1):aquint[9]
    quintIND[8] = (aquint[9]+1):aquint[6]
    @show quintIND

    # quintIND is ranges
    MPC = zeros(dist)
    MPA = zeros(MPC)
    MPH = zeros(MPC)
    MPL = zeros(MPC)
    x1a = zeros(MPC)
    gh = (mu_female*gh_f + mu_male*gh_m)
  #  gh = mu_male*gh_m
    Ea = A
    β = 0.98 ## FIX ME
    α = (1 - β)*Ea

    for s_nx in 1:n_s, ret_nx in 1:n_ret, emp_m_nx in 1:n_emp_m, emp_f_nx in 1:n_emp_f, d_nx in 1:n_d, a_nx in 2:n_a
      #sval = () not possible to define one sval for household. composition matters.
      IND1 = (a_nx-1, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx)
      IND2 = (a_nx, s_nx, ret_nx, emp_m_nx, emp_f_nx, d_nx)
      MPC[IND1...] = ((gc[IND2...] - gc[IND1...])/
      (agrid[a_nx] - agrid[a_nx - 1]))
      MPA[IND1...] = ((gval[IND2...] - gval[IND1...])/
      (agrid[a_nx] - agrid[a_nx - 1]))
      MPH[IND1...] = ((gh[IND2...] - gh[IND1...])/
      (agrid[a_nx] - agrid[a_nx - 1]))
      MPL[IND1...] = s_vals[s_nx, d_nx]*((gh[IND2...] - gh[IND1...])/
      (agrid[a_nx] - agrid[a_nx - 1]))
      x1a[IND1...] = α + (β - 1)*agrid[a_nx - 1]
    end
    MPA = MPA - 1
    # this misses a spike of people at the end of the asset distribution.
    table1 = zeros(length(quintIND), 4)
    for q_nx in 1:length(quintIND)
      qIND = quintIND[q_nx]
      table1[q_nx, 1] = sum(x1a[qIND, :].* dist[qIND, :]) / sum(dist[qIND, :])
      table1[q_nx, 2] = sum(MPA[qIND, :].* dist[qIND, :]) / sum(dist[qIND, :])
      table1[q_nx, 3] = sum(MPC[qIND, :].* dist[qIND, :]) / sum(dist[qIND, :])
      table1[q_nx, 4] = sum(MPH[qIND, :].* dist[qIND, :]) / sum(dist[qIND, :])
    end
    table2 = zeros(length(quintIND) + 1, 4)
    for q_nx in 1:(length(quintIND) + 1)
      if q_nx == length(quintIND) + 1
        IND = (1:(n_a - 1), :, :, :, :, :)
      else
        qIND = quintIND[q_nx]
        IND = (qIND, :, :, :, :, :)
      end
      table2[q_nx, 1] = 100sum(x1a[IND...].*MPA[IND...].* dist[IND...])/A
      table2[q_nx, 2] = 100sum(x1a[IND...].*MPC[IND...].* dist[IND...])/C
      table2[q_nx, 3] = 100sum(x1a[IND...].*MPH[IND...].* dist[IND...])/H
      table2[q_nx, 4] = 100sum(x1a[IND...].*MPL[IND...].* dist[IND...])/L
    end
    table3 = zeros(6, 4)
    for s_nx in 1:6
      IND = (1:(n_a - 1), s_nx, 1, :, :, :)
      if s_nx == 5
        IND = (1:(n_a - 1), :, 2, :, :, :)
      end
      if s_nx == 6
        IND = (1:(n_a - 1), :, :, :, :, :)
      end
      table3[s_nx, 1] = 100sum(x1a[IND...].*MPA[IND...].* dist[IND...])/A
      table3[s_nx, 2] = 100sum(x1a[IND...].*MPC[IND...].* dist[IND...])/C
      table3[s_nx, 3] = 100sum(x1a[IND...].*MPH[IND...].* dist[IND...])/H
      table3[s_nx, 4] = 100sum(x1a[IND...].*MPL[IND...].* dist[IND...])/L
    end
    println(table1)
    println(table2)
    println(table3)
    wealthquint2 = 100*Float64[sum(agrid[ind] .* dist[ind, :] ) for ind in quintIND]/A # Wealth qunitiles.
    println(wealthquint2)
  end

  return (CH, wealthquint, incquint, demp, noassets, fquint, Y, medy, avg_H, medINC)
end
